"""Particle system for JetPack Battle.

Accepts sprites (which contain a isjetting property)
and a sprite group to add new particle sprites to.
"""

import math
import pygame
import random

#--- CONSTANTS (these can be tweaked)
NUM_PARTICLES = 3
SPAWN_RANGE = (12, 24)
ANGLE_RANGE = (0.3 * math.pi, 0.7 * math.pi)

#--- CLASSES (sprites, etc.)

class Particle(pygame.sprite.Sprite):
    """Shortlived Sprite for Particle effect."""

    def __init__(self, pos, angle):
        """"""
        pygame.sprite.Sprite.__init__(self)

        self.rect = pygame.Rect(pos, (4, 4))
        self.image = pygame.Surface((4, 4))
        self.image = self.image.convert()
        self.image.fill((255, random.randint(0, 255), 255 * random.randint(0, 1)))
        self.hv = 5.0 * math.cos(angle)
        self.vv = 5.0 * math.sin(angle)
        self.pos = pos
        self.rect.center = pos
        self.distance = random.randint(400, 6400)

    def update(self):
        """Kill when far enough from start"""
        self.rect.centerx += self.hv
        self.rect.centery += self.vv

        if ((self.pos[0] - self.rect.centerx)**2 + (self.pos[1] - self.rect.centery)**2) > self.distance:
            self.kill()



#--- FUNCTIONS (these do the heavy lifting)
# ideally this would be more of a message passing system...

def jetparticles(sprite, particles):
    """Create particles to be added.

    Arguments:
    sprite: object with position, isleft, and isjetting
    particles: SpriteGroup basically
    """
    if sprite.isjetting:
        # get x coordinate based on left/right orientation
        xcoord = random.randint(*SPAWN_RANGE)
        if sprite.isleft:
            xcoord = sprite.rect.left + xcoord
        else:
            xcoord = sprite.rect.right - xcoord

        # create specified number of particles
        for i in range(NUM_PARTICLES):
            particles.add(Particle((xcoord, sprite.rect.centery),
                random.uniform(*ANGLE_RANGE)))

