"""Module to hold game sprites."""

import math
import random
import pygame
import pygame.sprite

import util

class Player(pygame.sprite.Sprite):
    """moves a clenched fist on the screen, following the mouse"""

    def __init__(self, isleft=False):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer

        self.isleft = isleft
        self.velocity = 0
        self.isjetting = False
        self.hp = 100

        # flip image if on the other side
        if isleft:
            self.image, self.rect = util.load_png('red.png')
            self.image = pygame.transform.flip(self.image, 1, 0)
            self.rect.center = (100, 100)
        else:
            self.image, self.rect = util.load_png('blue.png')
            self.rect.center = (540, 100)

    def update(self):
        """Move the fist based button press."""

        # create gravity
        self.rect.bottom += self.velocity
        if self.rect.bottom < 480:
            self.velocity += 0.2
        else:
            self.velocity = 0
            self.rect.bottom = 480

        # use jetpack to fight gravity
        if self.rect.top < 0:
            self.isjetting = False
            if self.velocity < 0:
                self.velocity = 0.0
        if self.isjetting:
            self.velocity -= 0.3

    def jet(self):
        "returns true if the fist collides with the target"
        if not self.isjetting:
            self.isjetting = True

    def unjet(self):
        "called to pull the fist back"
        self.isjetting = False

    def fire(self, target):
        """Fire weapon at enemy."""
        hv = -10.0
        if self.isleft:
            hv *= -1.0
        return Bullet(target, self.rect.center, hv, vv=self.velocity)

    def hit(self):
        """"""
        self.hp -= 10
        if self.hp <= 0:
            self.kill()


class Bullet(pygame.sprite.Sprite):
    """Hit things."""

    def __init__(self, target, pos, hv, vv):
        """"""
        pygame.sprite.Sprite.__init__(self)

        self.image, self.rect = util.load_png('bullet.png')
        self.target = target
        self.hv = hv
        self.vv = vv
        self.rect.center = pos

    def update(self):
        """"""
        self.rect.centerx += self.hv
        self.rect.centery += self.vv

        # create gravity
        if self.rect.bottom < 480:
            self.vv += 0.2

        # delete self when hit person/go off screen
        if self.rect.colliderect(self.target.rect):
            self.target.hit()
            self.kill()
        elif self.rect.centerx > 640 and self.rect.centerx < 0:
            self.kill()
